import chimera, numpy
from chimera import runCommand
from numpy import array, arccos, linalg, zeros

from StructBio.Scenographics.solids import Solids
from StructBio.LeastSquares.inertialAxes import InertialAxes
from StructBio.SSEs.tubesForSSEs import TubesForSSEs
from StructBio.LeastSquares.planes import Planes


#==============================================================================
# NOTE: This code is only suitable for mechanosensitive channels 2OAU and 2VV5.
# Note also that the primary axis derived from the computation of the inertial axes
# will be the axis of symmetry because of the overall shape of the protein.
# This is because the longest dimension of the molecule (height of the channel)
# is along this axis of symmetry.
#==============================================================================

def anglesForMscS(prot):
    ################## Part 1:  Put in three extra visual components ###############
    # 1) Put tubes into all the helices:
    #       Note high tolerance value to avoid recursive tubing.
    helixTubes = TubesForSSEs(prot, tolerance = 4.0, minLength = 6, chainId = "C", strands = False)

    # The next two visual components will need some calculations that define three points
    # along the main axis of the protein. We start by generating the inertial axis for the protein.
    caAtoms_L = []
    for r in prot.residues:
        if r.findAtom("CA"):
            caAtoms_L.append(r.findAtom("CA"))
    mainAxis = InertialAxes(caAtoms_L) # NB: This, by itself, does not produce any visual component.
    # We project three atom coordinates onto this inertial axes to get points needed to specify
    # the remaining visual components:
    # (Residue positions 0, 35, and 250 were determined by visual inspection.)
    tubeTopPt = mainAxis.projectionOnAxis(prot.findResidue(0).findAtom("CA"))   
    diskCenterPt = mainAxis.projectionOnAxis(prot.findResidue(35).findAtom("CA"))  
    tubeBotPt = mainAxis.projectionOnAxis(prot.findResidue(250).findAtom("CA"))     

    # 2) Put in the tube for the main axis:
    axisTubeDisp = Solids()
    axisTubeDisp.addTube(tubeBotPt, tubeTopPt, 1.0, (.2, .3, .7))
    axisTubeDisp.display()

    # 3) Put in the disk perpendicular to the main axis:
    diskDisp = Solids()
    diskDisp.addDisk(diskCenterPt, mainAxis.primaryAxis, 0.2, 40., (0.2, 0.6, .4))
    diskDisp.display()

    ######################## Part 2: Do the angle calculations ######################
    # Note that we can access the helixTubes object to get the startPoint and endPoint
    # coordinates for the axis of a helix.  These points can be used as arguments for 
    # the angleToPLane method of the Planes class.
    pi = numpy.pi
    # Start by generating a Planes object that is parallel to the visual diskDisplay.
    diskPlane = Planes(diskCenterPt, mainAxis.primaryAxis)

    # Get start and end points of the transmembrane helices and compute the required angles:
    # Use the helices above the cytoplasmic membrane that are in Chain C because this chain
    # has three distinct helices at the beginning of the chain (unlike chain A).
    retVal = zeros(3, float)
    for ix in range(3):        
        sPt = helixTubes.axes.inertialAxes_L[ix].startPoint
        ePt = helixTubes.axes.inertialAxes_L[ix].endPoint
        retVal[ix] = diskPlane.angleToPlane(sPt, ePt)
    return retVal

#==============================================================================
# MAINLINE:
#==============================================================================
runCommand("set bg_color white")

models_L = chimera.openModels.open("2OAU", type="PDB") # Get closed conformation.
p2OAU = models_L[0]
angles2OAU_A = anglesForMscS(p2OAU)

raw_input("Key Enter to continue with next protein.")
# Change the next two lines into comments if you want to see both proteins in the window.
# If both proteins are displayed they will not be overlapped. In the next chapter we
# will develop code to do such an overlap.
for om in chimera.openModels.list():
    chimera.openModels.close(om)

models_L = chimera.openModels.open("2VV5", type="PDB") # Get open conformation
p2VV5 = models_L[0]
angles2VV5_A = anglesForMscS(p2VV5)

runCommand("window")

print "\nAngles between helix axis and membrane plane (closed conformation):"
for i in range(3):
    print "Helix[%1d]" % i, "angle = %6.3f" % angles2OAU_A[i]

print "\nAngles between helix axis and membrane plane (open conformation):"
for i in range(3):
    print "Helix[%1d]" % i, "angle = %6.3f" % angles2VV5_A[i]

print "\nAngle changes in going from closed to open conformation: "
for i in range(3):
    print  "Helix[%1d]" % i, "angle change = %6.3f" % (angles2VV5_A[i] - angles2OAU_A[i])











