import chimera, Tkinter as tk, tkFileDialog, Image
from chimera.baseDialog import ModelessDialog
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
from matplotlib.pyplot import imshow

#-------------------------------------------------------------------------
def preProline(r):
    if not r.molecule.residueAfter(r): return False
    rNext = r.molecule.residueAfter(r)
    if rNext.id.chainId != r.id.chainId: return False
    if rNext.type == "PRO": return True
    return False

class RamaPlotDialog(ModelessDialog):
    buttons = ("Save", "Close")
    help = False    # Leave this out to get a Help button.
    title = "Ramachandran Plot for Residues"
    
    #-----------------------------------------------------------------------
    def fillInUI(self, parent):
        self.prot = None
        self.residuePhiPsi_D = None
        self.rBtn = {}
        self.fig = None
        #
        # Setting up button to get list of proteins:
        buttonFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        buttonFrame.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.W)
        
        buttonText = tk.Label(buttonFrame, text =  " Process list of proteins:   ")
        buttonText.grid(row = 0, column = 0, pady = 4, sticky = tk.W)
        
        self.btnGetProtList = tk.Button(buttonFrame, text = " Get List ",
                                        command = lambda: self.btnGetListFunc())
        self.btnGetProtList.grid(row = 0, column = 1, padx = 4, pady = 4)
        #
        # Set up radio buttons as specified by a list:
        rButtonsFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        rButtonsFrame.grid(row = 1, column = 0, columnspan = 8, padx = 4, pady = 4, sticky = tk.W)
        
        rButtonText = tk.Label(rButtonsFrame, text =  " Select residue type:  ")
        rButtonText.grid(row = 0, column = 0, pady = 4, columnspan = 2, sticky = tk.W)
        
        
        self.rBtnResult = tk.StringVar() # Use tk.IntVar() here for integer result values.
        radioBtn_L = [("ALA", 1, 0), ("ARG", 1, 1), ("ASP", 1, 2), ("ASN", 1, 3),
                      ("CYS", 1, 4), ("GLU", 1, 5), ("GLN", 1, 6), ("GLY", 2, 0),
                      ("HIS", 2, 1), ("ILE", 2, 2), ("LEU", 2, 3), ("LYS", 2, 4),
                      ("MET", 2, 5), ("PHE", 2, 6), ("PRO", 3, 0), ("SER", 3, 1),
                      ("THR", 3, 2), ("TRP", 3, 3), ("TYR", 3, 4), ("VAL", 3, 5),
                      ("Pre-Proline", 3, 6)]
        
        for text, rw, cl in radioBtn_L:
            self.rBtn[text] = tk.Radiobutton(rButtonsFrame, text = text, variable = self.rBtnResult,
                                             value = text, state = tk.DISABLED, 
                                             command = lambda: self.rBtnFunc(parent))
            self.rBtn[text].grid(row = rw, column = cl, padx = 8, sticky = tk.W)

        self.rBtn["ALA"].select()
            
    #-------------------------------------------------------------------------
    # Callback functions needed for Get List button:
    def btnGetListFunc(self):
        # Launch dialog to get path of PDB ID list:
        filePathName = tkFileDialog.askopenfilename()
        # Initialize dictionary to hold empty lists corresponding to each residue.
        # Entries in a list will be tuples with format (SSEcode, phi, psi).
        # The SSEcode will be "H" for helix, "S" for strand, or "C" for coil.
        self.residuePhiPsi_D = {"ALA": [], "ARG": [], "ASP": [], "ASN": [], "CYS": [],
                                "GLU": [], "GLN": [], "GLY": [], "HIS": [], "ILE": [],
                                "LEU": [], "LYS": [], "MET": [], "PHE": [], "PRO": [],
                                "SER": [], "THR": [], "TRP": [], "TYR": [], "VAL": [],
                                "Pre-Proline": []}

        # Read file containing list of PDB identifiers:
        listFileHandle = file(filePathName)
        listFileHandle.readline()    # Skip first line with file information.
        pdbID = listFileHandle.readline()
        while pdbID:
            print ("pdb ID: %s    chain ID: %s" % (pdbID[0:4], pdbID[4]))
            prot_L = chimera.openModels.open(pdbID[0:4], type = "PDB")
            prot = prot_L[0]
            chainID = pdbID[4]

            # Stash the phi, psi angles into the dictionaries:
            for r in prot.residues:
                if r.id.chainId != chainID: continue
                if not(r.phi and r.psi): continue # Skip if residue has no phi, psi attributes.
                if r.type not in self.residuePhiPsi_D.keys(): continue
                if r.isHelix: self.residuePhiPsi_D[r.type].append(("H", r.phi, r.psi))
                if r.isStrand: self.residuePhiPsi_D[r.type].append(("S", r.phi, r.psi))
                if not (r.isHelix or r.isStrand): self.residuePhiPsi_D[r.type].append(("C", r.phi, r.psi))
                if preProline(r):
                    if r.isHelix: self.residuePhiPsi_D["Pre-Proline"].append(("H", r.phi, r.psi))
                    if r.isStrand: self.residuePhiPsi_D["Pre-Proline"].append(("S", r.phi, r.psi))
                    if not (r.isHelix or r.isStrand): self.residuePhiPsi_D["Pre-Proline"].append(("C", r.phi, r.psi))                    

            pdbID = listFileHandle.readline()   # Go for next pdbID in the list.
            chimera.openModels.close(prot_L)
            
        listFileHandle.close()  # Close the list file.
        
        for btnTxt in self.rBtn.keys():
            self.rBtn[btnTxt].configure(state = tk.NORMAL)

        self.rBtn["ALA"].invoke()
            
    #-----------------------------------------------------------------------
    def Save(self):
        # Save file if the prot is not None.
        if self.fig != None:
            self.fig.savefig(self.selectedRes + "_RamachandranPlot", dpi = self.fig.dpi)

    #-----------------------------------------------------------------------
    # Callback function needed for radio buttons:
    def rBtnFunc(self, parent):
        self.selectedRes = self.rBtnResult.get()
        
        self.fig = Figure(figsize = (6,5), dpi = 100)

        ax = self.fig.add_subplot(111, aspect = 'equal')
        ax.set_title("Ramachandran Plot for " + self.selectedRes, fontsize = 14)
        ax.set_xlabel("PHI (degrees)", fontsize = 12)
        ax.set_ylabel("PSI (degrees)", fontsize = 12)
        ax.set_xlim([-180, 180])
        ax.set_ylim([-180, 180])
        ax.set_xticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
        ax.set_yticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
        img = Image.open('croppedRamachandranBackGround.png')
        ax.imshow(img, extent = (-180, 180, 180, -180))
        ax.grid(True, linestyle = '-', color = '0.75')
        ax.hlines(0, -180, 180, colors='k', linestyles='dashed')
        ax.vlines(0, -180, 180, colors='k', linestyles='dashed')

        # Initialization:
        # List indicies: 0 for Helix, 1 for Strand, and 2 for Coil.
        colors_D = {"H": 'k', "S": 'g', "C": 'b'} # Marker colors for Helix, Strand, and Coil.
        shapes_D = {"H": 'o', "S": 's', "C": 'd'} # Marker shapes for Helix, Strand, and Coil.

        for entry in self.residuePhiPsi_D[self.selectedRes]:
            if len(entry) == 0: continue
            ax.scatter(entry[1], entry[2], s = 20,
                       color = colors_D[entry[0]], marker = shapes_D[entry[0]])

        #-------------------------------------------------------------------------
        # Set up a canvas and put the Figure into it.
        canvasFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        canvasFrame.grid(row = 2, column = 0, columnspan = 8, padx = 4, pady = 4, sticky = tk.E)

        canvas = FigureCanvasTkAgg(self.fig, master = canvasFrame)
        canvas.get_tk_widget().grid(row = 0, column = 0)

        # Set up legend text.
        legendText = tk.Label(canvasFrame, text = "Helix: black circle.    Strand: green square.    Coil: blue diamond.")
        legendText.grid(row = 1, column = 0, columnspan = 8, padx = 4, sticky = tk.W)

#==============================================================================
# Mainline:

RamaPlotDialog()
