import chimera, Tkinter as tk, tkMessageBox
from chimera import runCommand, selection
from chimera.baseDialog import ModelessDialog

class sideChainControlPanel(ModelessDialog):
    buttons = ("Close")
    help = False    # Leave this out to get a Help button.
    title = "Side-Chain Control Panel"

    def fillInUI(self, parent):
        self.res = None
        # Text to be displayed at the top of the dialog window:
        topText = tk.Label(parent, text =  " Select an atom to choose a side-chain. Then click Accept.",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 4, pady = 4, sticky = tk.W)

        # Button to accept atom that specifies a side chain:
        acceptButton = tk.Button(parent, text = " Accept ", command = self.acceptSelection)
        acceptButton.grid(row = 1, column = 1, padx = 4, pady = 4, sticky = tk.W)

        # Label to describe the chosen residue:
        self.resInfoVar = tk.StringVar()
        resInfoLabel = tk.Label(parent, textvariable = self.resInfoVar, width = 20,
                               bg = "white", bd = 2, relief = tk.GROOVE)
        resInfoLabel.grid(row = 1, column = 2, padx = 4, pady = 4, sticky = tk.W)

        #-------------------------------------------------------------------------
        # Setting up the scales:
        scaleFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        scaleFrame.grid(row = 4, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W)
        
        scaleText = tk.Label(scaleFrame, text =  " Torsion angles for this side chain:  ")
        scaleText.grid(row = 0, column = 0, columnspan = 2, pady = 2, sticky = tk.W)

        # Declare all dictionaries.  Keys will be torsion angles:
        keys_L = ["chi1", "chi2", "chi3", "chi4"]
        self.scaleVar_D = {}; self.scale_D = {}; self.chiOutVar_D = {}; chiOut_D = {}; self.increasing_D = {}
        self.initialChiVals_D = {}

        for k in keys_L:
            self.increasing_D[k] = True
            
            # Label with name of torsion angle:
            tk.Label(scaleFrame, text = " " + k + ": ", font = ("Helvetica", 11, "normal")) \
                                 .grid(row = int(k[3]), column = 0, pady = 2, sticky = tk.W)
            # Scale:
            self.scaleVar_D[k] = tk.DoubleVar()
            self.scale_D[k] = tk.Scale(scaleFrame, variable = self.scaleVar_D[k], orient = tk.HORIZONTAL,
                                       command = lambda event, arg = k: self.updateValue(arg),
                                       length = 360, from_ = 0.0, to = 360.0, resolution = 0.1,
                                       troughcolor = "gray", showvalue = False)
            self.scale_D[k].grid(row = int(k[3]), column = 1, sticky = tk.W)
            self.scale_D[k].set(180.0)
            self.scale_D[k].config(state = tk.DISABLED)

            # Label that displays the chi angle:
            self.chiOutVar_D[k] = tk.StringVar()
            chiOut_D[k] = tk.Label(scaleFrame, textvariable = self.chiOutVar_D[k], width = 6,
                                   bg = "white", bd = 2, relief = tk.GROOVE)
            chiOut_D[k].grid(row = int(k[3]), column = 2, padx = 4, pady = 4, sticky = tk.E)
            
        #-------------------------------------------------------------------------
        # Label and Button to reset torsions to initial values:
        tk.Label(parent, text =  " Reset torsion angles to initial values:  ") \
                         .grid(row = 5, column = 1, columnspan = 2, pady = 2, sticky = tk.W)

        self.resetButton = tk.Button(parent, text = " Reset ", state = tk.DISABLED,
                                     command = self.resetTorsionAngles)
        self.resetButton.grid(row = 5, column = 2, padx = 4, pady = 4, sticky = tk.E)
        
        
    #=============================================================================
    # CALLBACK functions:
    #-------------------------------------------------------------------------
    # Callback function needed for Scale widget:
    def acceptSelection(self):
        atm_L = selection.currentAtoms()
        if len(atm_L) == 0:
            map(self.disableScale, ("chi1", "chi2", "chi3", "chi4"))
            self.resInfoVar.set("")
            self.resetButton.configure(state= tk.DISABLED)
            return
        # If we get this far, we have a residue.
        self.resetButton.configure(state= tk.NORMAL)
        self.res = atm_L[0].residue

        runCommand("sel :" + str(self.res.id.position) + "." +
                   self.res.id.chainId + " za<8.0; focus sel; ~sel")
        self.resInfoVar.set(self.res.type + " " + str(self.res.id.position) +
                            "." + self.res.id.chainId + "  in  " + self.res.molecule.name)       

        if self.res.chi1 != None: self.enableScale("chi1", self.res.chi1)
        else: self.disableScale("chi1")
        if self.res.chi2 != None: self.enableScale("chi2", self.res.chi2)
        else: self.disableScale("chi2")
        if self.res.chi3 != None: self.enableScale("chi3", self.res.chi3)
        else: self.disableScale("chi3")
        if self.res.chi4 != None: self.enableScale("chi4", self.res.chi4)
        else: self.disableScale("chi4")

        if sideChainInCycle(self.res):
            tkMessageBox.showwarning("NOTE: ", "You cannot change torsion angles \n for side chains in a cycle.")  

    def enableScale(self, arg, chiVal):
        self.scaleVar_D[arg].set(180.0 + chiVal)                # Set scale to continue with chi value.
        self.increasing_D[arg] = True                           # Initialize to " values increasing" mode.
        self.chiOutVar_D[arg].set("%7.1f " % chiVal)            # Initialize scale Label.
        self.initialChiVals_D[arg] = chiVal                     # Save current torsion (to be used as "initial" value)
        if not sideChainInCycle(self.res):                      # Chimera disallows changing torsions if sidechain is part of cycle.
            self.scale_D[arg].config(state = tk.NORMAL)         # Enable it for user interaction.
            self.scale_D[arg].config(troughcolor = "goldenrod1")# Change color to indicate being enabled.
        else:
            self.scale_D[arg].config(state = tk.DISABLED)       # Enable it for user interaction.
            self.scale_D[arg].config(troughcolor = "gray")      # Change color to indicate being enabled.
            self.resetButton.configure(state= tk.DISABLED)

    def disableScale(self, arg):
        self.scaleVar_D[arg].set(180.0)                         # Initialize scale.
        self.scale_D[arg].config(state = tk.DISABLED)           # Enable it for user interaction.
        self.scale_D[arg].config(troughcolor = "gray")          # Change color to indicate being enabled.
        self.increasing_D[arg] = True                           # Initialize to " values increasing" mode.
        self.chiOutVar_D[arg].set("")                           # Clear scale Label.

    # Callback function needed for Scale widget:
    def updateValue(self, arg):
        if not self.res: return
        value = self.scaleVar_D[arg].get()
        if value == 0.0 or value == 360: self.increasing_D[arg] = not self.increasing_D[arg]
        if self.increasing_D[arg]:
            self.chiOutVar_D[arg].set("%7.1f " % (value - 180.0))
            self.changeChi(arg, value - 180.0)
        if not self.increasing_D[arg]:
            self.chiOutVar_D[arg].set("%7.1f " % (180.0 - value))
            self.changeChi(arg, 180.0 - value)

    def changeChi(self, arg, newVal):
        if arg == "chi1": self.res.chi1 = newVal
        if arg == "chi2": self.res.chi2 = newVal
        if arg == "chi3": self.res.chi3 = newVal
        if arg == "chi4": self.res.chi4 = newVal

    def resetTorsionAngles(self):
        if sideChainInCycle(self.res): return
        if self.res.chi1 != None:
            self.enableScale("chi1", self.initialChiVals_D["chi1"])
            self.res.chi1 = self.initialChiVals_D["chi1"]
        if self.res.chi2 != None:
            self.enableScale("chi2", self.initialChiVals_D["chi2"])
            self.res.chi2 = self.initialChiVals_D["chi2"]
        if self.res.chi3 != None:
            self.enableScale("chi3", self.initialChiVals_D["chi3"])
            self.res.chi3 = self.initialChiVals_D["chi3"]
        if self.res.chi4 != None:
            self.enableScale("chi4", self.initialChiVals_D["chi4"])
            self.res.chi4 = self.initialChiVals_D["chi4"]
        
def sideChainInCycle(r):
    if r.type == "PRO": return True
    if r.type == "CYS" and "SG" in [a.name for a in r.findAtom("SG").neighbors]:
        return True
    return False
    
#===============================================================================
# Mainline for testing:        
wb = sideChainControlPanel()
